<#

    .SYNOPSIS
    Library file

    .DESCRIPTION

    .PARAMETER

    .EXAMPLE

    .NOTES
    Author: Morten Rnborg
    Date: 18-07-2018
    Last Updated: 21-07-2018

#>

################################################
Function Write-Log
{
    param 
    (
        [Parameter(Mandatory=$true, HelpMessage="Provide a message")][string]$LogOutput,
        [Parameter(Mandatory=$true, HelpMessage="Provide the function name")][string]$FunctionName,
        [Parameter(Mandatory=$false, HelpMessage="Provide the scriptlinenumber")][string]$ScriptLine,
        [Parameter(Mandatory=$false, HelpMessage="Provide path, default is .\Logs")][string]$Path,
        [Parameter(Mandatory=$false, HelpMessage="Provide name for the logs")][string]$Name,
        [Parameter(Mandatory=$false, HelpMessage="Provide level, 1 = default, 2 = warning 3 = error")][ValidateSet(1, 2, 3)][int]$LogLevel = 1
    )

    #If the scriptline is not defined then use from the invocation
    If(!($ScriptLine)){
        $ScriptLine = $($MyInvocation.ScriptLineNumber)
    }

    if($LogOutput){

        #Date for the lognaming
        $FullLogName = ($Path + "\" + $Name + ".log")
        $FullSecodaryLogName = ($FullLogName).Replace(".log",".lo_")

        #If the log has reached over 20 mb then rename it
        if(Test-Path $FullLogName){
            if((Get-Item $FullLogName).Length -gt 5000kb){
                if(Test-Path $FullSecodaryLogName){
                    Remove-Item -Path $FullSecodaryLogName -force
                }
                Rename-Item -Path $FullLogName -NewName $FullSecodaryLogName
            }
        }

        #First check if folder/logfile excist, if not then create it
        if(!(test-path $Path)){
            New-Item -ItemType Directory -Force -Path $Path -ErrorAction SilentlyContinue
        }

        #Get current date and time to write to log
        $TimeGenerated = "$(Get-Date -Format HH:mm:ss).$((Get-Date).Millisecond)+000"

        #Construct the logline format
        $Line = '<![LOG[{0}]LOG]!><time="{1}" date="{2}" component="{3}" context="" type="{4}" thread="" file="">'

        #Define line
        $LineFormat = $logOutput, $TimeGenerated, (Get-Date -Format MM-dd-yyyy), "$($FunctionName):$($Scriptline)", $LogLevel

        #Append line
        $Line = $Line -f $LineFormat

        #Write log
        try {
            Write-Host ("[$($FunctionName):$($Scriptline)]" + $logOutput)
            $Line | Out-File -FilePath ($Path + "\" + $Name + ".log") -Append -NoClobber -Force -Encoding 'UTF8' -ErrorAction 'Stop'
        }
        catch {
            Write-Host "$_"
        }
    }
}


# Function to create a random time stamp
function Random-StartTime
{
	[string]$RandomHour = (Get-Random -Maximum 12) 
	[string]$RandomMinute = (Get-Random -Maximum 59)
	[string]$RandomStartTime = $RandomHour + ":" + $RandomMinute
	return $RandomStartTime
}

function Get-CMApplicationsInFolder{

    Param(
        $FolderName,
        $SCCMServer,
        $SCCMSiteCode,
        $ParentContainerNodeID = 0 # 0 is the root ID
    )

    #Get folder ID
    $FolderID = (Get-WmiObject -ComputerName $SCCMServer -Namespace "ROOT\SMS\Site_$SCCMSiteCode" -Query "Select name,containernodeid from SMS_ObjectContainerNode where ObjectType='6000' and ParentContainerNodeID = '$ParentContainerNodeID' and name ='$FolderName'").containernodeid
    
    #Get instance key
    $Instancekeys = (Get-WmiObject -ComputerName $SCCMServer -Namespace "ROOT\SMS\Site_$SCCMSiteCode" -Query "select InstanceKey from SMS_ObjectContainerItem where ObjectType='6000' and ContainerNodeID='$FolderID'").InstanceKey

    #Get all applications
    $Applications = (Get-WmiObject -ComputerName $SCCMServer -Namespace "ROOT\SMS\Site_$SCCMSiteCode" -Query "select LocalizedDisplayName,ModelName from SMS_Applicationlatest")

    #Return only the applications in the given folder
    Return ($Applications | Where-Object{$_.ModelName -in $Instancekeys}).LocalizedDisplayName
    
}


function Get-MsiProperty
{
	param(
		[string]$Path,
		[string]$Property
	)
	    
	function Get-Property($Object, $PropertyName, [object[]]$ArgumentList)
	{
		return $Object.GetType().InvokeMember($PropertyName, 'Public, Instance, GetProperty', $null, $Object, $ArgumentList)
	}
	 
	function Invoke-Method($Object, $MethodName, $ArgumentList)
	{
		return $Object.GetType().InvokeMember($MethodName, 'Public, Instance, InvokeMethod', $null, $Object, $ArgumentList)
	}
	 
	$ErrorActionPreference = 'Stop'
	Set-StrictMode -Version Latest
	 
	$msiOpenDatabaseModeReadOnly = 0
	$Installer = New-Object -ComObject WindowsInstaller.Installer
	 
	$Database = Invoke-Method $Installer OpenDatabase @($Path, $msiOpenDatabaseModeReadOnly)
	 
	$View = Invoke-Method $Database OpenView  @("SELECT Value FROM Property WHERE Property=$Property")
	 
	Invoke-Method $View Execute
	 
	$Record = Invoke-Method $View Fetch
	if ($Record)
	{
		Write-Output(Get-Property $Record StringData 1)
	}
	 
	Invoke-Method $View Close @( )
	Remove-Variable -Name Record, View, Database, Installer
	 
}
function Test-MimimumAccessRights{
    param(
        $UserName,
        $Path
    )

    #Define variables
    $PSPath =("Microsoft.PowerShell.Core\FileSystem::" + $Path) 

    #Get all the memberships of the user specified
    [array]$GroupMemberShips = (New-Object System.DirectoryServices.DirectorySearcher("(&(objectCategory=User)(samAccountName=$UserName))")).FindOne().GetDirectoryEntry().memberOf

    #Get all FullControl with inheritance
    $IdentityWithFullControlAndInheritance = ((Get-Acl $PSPath).Access | Where-Object{($_.InheritanceFlags -eq "ContainerInherit, ObjectInherit") -and (($_.FileSystemRights -eq "FullControl" -or $_.FileSystemRights -eq "DeleteSubdirectoriesAndFiles, Modify, Synchronize"))}).IdentityReference

    ForEach($Identity in $IdentityWithFullControlAndInheritance){

        #Define variables
        $FormattedIdentity = ($Identity).ToString().Split("\")[1]

        #Check if there is directly user access
        If($UserName -eq $FormattedIdentity){
            Return $true
        }

        #Check if there is a membership match
        ForEach($GroupMemberShip in $GroupMemberShips){

            #If there is a membership match return true
            If(($GroupMemberShip -like "*$FormattedIdentity*")-and ($FormattedIdentity -notlike $null)){
                Return $true
            }
        }
    }

    #return false
    Return $false

}

Function Get-ChildItemToDepth {
    Param(
    [string]$Path,
    [string]$Filter,
    [int]$ToDepth = 3,
    [int]$CurrentDepth
    )
    $CurrentDepth++
    Get-ChildItem $Path -Filter $Filter | ForEach-Object{
        if($_.FullName -like $Filter){
            [array]$PathToFile += $_.FullName
            }
        If ($_.PsIsContainer) {
            If ($CurrentDepth -le $ToDepth){
                # Callback to this function
                Get-ChildItemToDepth -Path "Microsoft.PowerShell.Core\FileSystem::$($_.FullName)" -Filter $Filter -ToDepth $ToDepth -CurrentDepth $CurrentDepth
            }
        }
    }
    
    #return
    return $PathToFile
}
Function Import-ApplicationInfoFiles{
    Param(
        $Path
    )

    #Get autoapp files
    $AppsPath = Get-ChildItemToDepth -Path $Path -filter "*.*" -ToDepth 3 | Where-Object{$_ -like "*.autoapp"}

    #Define array
    $ApplicationInfo = @()
    
    #Put into array
    ForEach($AppPath in $AppsPath){

        #Import XML
        $AppInfo = Import-Clixml -Path "Microsoft.PowerShell.Core\FileSystem::$AppPath"
        $AppInfo | Add-Member -MemberType NoteProperty -Name "XMLFilePath" -Value  $AppPath -Force
        $ApplicationInfo += $AppInfo
    }

    #Return appinfo
    Return $ApplicationInfo
}

function Update-ApplicationInfoFile {
    param(
        $Path,
        $PropertyName,
        $Value
    )

    #Appinfo
    $AppInfo = Import-Clixml -Path "Microsoft.PowerShell.Core\FileSystem::$Path"

    #Add the member
    $AppInfo | Add-Member -MemberType NoteProperty -Name $PropertyName -Value  $Value -Force

    #Output the XML
    $AppInfo | Export-Clixml -Path ("Microsoft.PowerShell.Core\FileSystem::" + $Path) -Force

}