<#

    .SYNOPSIS
    SCCM manamgent script, to automatically create Application, collection and AD group. Only used for PSADT applications

    .DESCRIPTION

    .PARAMETER

    .EXAMPLE

    .NOTES
    Author: Morten Rnborg
    Date: 18-07-2018
    Last Updated: 08-10-2018

#>

################################################
#Requires -Modules ActiveDirectory

param(
    $SourceLocation,
    $OUPath,
    $DistributionPointGroup
)

# Dot Source library and logging variables
. "$PSScriptRoot\lib.ps1"
$LogLocation = "$PSScriptRoot\Logs\"
$LogName = "SCCM-AutoManagement"
$DriveName = "SCCMSite"

#Import module
Import-Module(Join-Path $(Split-Path $env:SMS_ADMIN_UI_PATH) ConfigurationManager.psd1)
Add-Type -AssemblyName System.Drawing #used for icon check
New-PSDrive -Name $DriveName -PSProvider "AdminUI.PS.Provider\CMSite" -Root "$env:computername" -Description "SCCM Site Mapping"

Write-Log -LogOutput ("***********************************************START SCRIPT***********************************************") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
Write-Log -LogOutput ("Running in the context of '$($env:USERNAME)'") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null

#Change location for the SCCM module to work
try {
    Set-Location($DriveName + ":") -ErrorAction Stop
}
catch {
    Write-Log -LogOutput ("$_ `nThis could be because the powershell console, in the SCCM console, has not been started by the service account before.:`nhttp://benef-it.blogspot.com/2013/04/solve-ps-drive-problem-with-sccm2012.html") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
    break
}
##*=============================================
##* VARIABLES LISTINGS
##*=============================================

$SCCMServer = (Get-CMSite).ServerName
$SCCMSiteCode = (Get-CMSite).SiteCode
$DomainNetbiosName = ((Get-WmiObject Win32_ComputerSystem).Domain).Split(".")[0]
$DeviceLimitingCollection = "All Systems"
$UserLimitingCollection = "All Users and User Groups"
$CollectionDevFolderName = "Application Deployments"
$CollectionDevFolderPath = $DriveName + ":" + "\DeviceCollection\$CollectionDevFolderName"
$CollectionUsrFolderName = "Application Deployments"
$CollectionUsrFolderPath = $DriveName + ":" + "\UserCollection\$CollectionUsrFolderName"
$ApplicationFolderName = "Auto Software Distribution"
$ApplicationFolderPath = $DriveName + ":" + "\Application\$ApplicationFolderName"  #Only supports folder in the root, see the Get-CMApplicationsInFolder function

#Deployment definitions
$Deployments = @{}
$Deployments["Available device install"] = @{Naming = "ava-ins-dev";DeploymentPurpose = "Available";DeployAction = "Install";Type = "Device"}
$Deployments["Required device install"] =  @{Naming = "req-ins-dev";DeploymentPurpose = "Required";DeployAction = "Install";Type = "Device"}
$Deployments["Required device uninstall"] =  @{Naming = "req-uni-dev";DeploymentPurpose = "Required";DeployAction = "Uninstall";Type = "Device"}
$Deployments["Available user install"] = @{Naming = "ava-ins-usr";DeploymentPurpose = "Available";DeployAction = "Install";Type = "User"}
$Deployments["Required user install"] =  @{Naming = "req-ins-usr";DeploymentPurpose = "Required";DeployAction = "Install";Type = "User"}
$Deployments["Required user uninstall"] =  @{Naming = "req-uni-usr";DeploymentPurpose = "Required";DeployAction = "Uninstall";Type = "User"}

##*=============================================
##* ENDVARIABLES LISTINGS
##*=============================================


Write-Log -LogOutput ("Loading data..") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

#Get all applications in the given folder in SCCM (we use this homemade function because the one in the module called Get-CMApplication function, dosen't support folder specification)
[array]$CurrentApplications = Get-CMApplicationsInFolder -FolderName $ApplicationFolderName -SCCMServer $SCCMServer -SCCMSiteCode $SCCMSiteCode
Write-Log -LogOutput ("Currently '$($CurrentApplications.Count)' applications in the SCCM Application folder '$($ApplicationFolderName)'. Applications:`n$($CurrentApplications -join "`n")")  -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

#Define an array
[array]$ApplicationsInSource = Import-ApplicationInfoFiles -Path "Microsoft.PowerShell.Core\FileSystem::$SourceLocation"

Write-Log -LogOutput ("Currently '$($ApplicationsInSource.Count)' applications in the SCCM Auto Application source folder '$($SourceLocation)'. Applications:`n$($ApplicationsInSource.Name -join "`n")")  -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null

#New applications
[array]$NewApplications = ($ApplicationsInSource | Where-Object{$_.AutoManageAction -eq "Create"})
[array]$DeleteApplications = ($ApplicationsInSource | Where-Object{$_.AutoManageAction -eq "Delete"})

Write-Log -LogOutput ("Applications to create is '$($NewApplications.Count)'")  -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
Write-Log -LogOutput ("Applications to delete is '$($DeleteApplications.Count)'")  -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

#Loop through every application to be created
ForEach($Application in $NewApplications){

    #Define variables
    $DetectionClauses = $null
    $LocalizedDisplayName = $Application.LocalizedDisplayName #name in Software Center
    $ApplicationName = $Application.Name #the unique name.
    $ApplicationSimpleName = $Application.SimpleName
    $Publisher = $Application.Publisher
    $ApplicationVersion = $Application.Version
    $SourcePath = $Application.SourcePath
    $IconLocation = ($Application.IconLocation)
    $InstallCommand = $Application.InstallCommand
    $UninstallCommand = $Application.UninstallCommand
    $Collection = ("$Publisher`_$ApplicationSimpleName`_$ApplicationVersion").Replace(" ","")
    $ADGroup = ("$Publisher`_$ApplicationSimpleName`_$ApplicationVersion").Replace(" ","")
    $XMLFilePath = $Application.XMLFilePath

    Write-Log -LogOutput ("********RUNNING PROCESS FOR '$($ApplicationName)'********")  -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

    #Creating the Application if not excisting
    if(!($ApplicationObject = Get-CMApplication -Name $ApplicationName)){

        # If the icon exist, create the application with that
        if(Test-Path ("Microsoft.PowerShell.Core\FileSystem::" + $IconLocation) ){

            #Check if iconsize meets the criteria
            $IconInfo = [System.Drawing.Image]::FromFile($IconLocation)
            if(($IconInfo.Height -le 250) -and ($IconInfo.Width -le 250)){

                # Create with icon
                $ApplicationObject = New-CMApplication -Name $ApplicationName -Publisher $Publisher -AutoInstall $true -SoftwareVersion $ApplicationVersion -LocalizedName $LocalizedDisplayName -IconLocationFile $IconLocation
            }else{

                # If the icon dosent meet the criteria
                $ApplicationObject = New-CMApplication -Name $ApplicationName -Publisher $Publisher -AutoInstall $true -SoftwareVersion $ApplicationVersion -LocalizedName $LocalizedDisplayName
            }
        }else{

            # If ther is no icon present
            $ApplicationObject = New-CMApplication -Name $ApplicationName -Publisher $Publisher -AutoInstall $true -SoftwareVersion $ApplicationVersion -LocalizedName $LocalizedDisplayName
        }
    }

    # Check if the applicationfolder exist
    if (!(Test-Path $ApplicationFolderPath))
    {
        Write-Log -LogOutput ("Application folder not existing, creating it..") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
        New-Item $ApplicationFolderPath
    }

    # Move the application to folder
    Move-CMObject -FolderPath $ApplicationFolderPath -InputObject $ApplicationObject

    # Check if the collection folder exist
    if (!(Test-Path $CollectionDevFolderPath))
    {
        Write-Log -LogOutput ("Device collection folder not existing, creating it..") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
        New-Item $CollectionDevFolderPath
    }

    # Check if the collection folder exist
    if (!(Test-Path $CollectionUsrFolderPath))
    {
        Write-Log -LogOutput ("User collection folder not existing, creating it..") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
        New-Item $CollectionUsrFolderPath
    }

    #Create collection, AD group and query rule for
    ForEach($Deployment in $Deployments.GetEnumerator()){

        # Define variables
        $Naming = $Deployment.Value.Naming
        $Type = $Deployment.Value.Type
        $CollectionName = ("$Collection`_$Naming") # If you change this, also change in the create deployment part
        $ADGroupName = ("$ADGroup`_$Naming")
        $NumberOfCharactersToRemove = $null
        while($ADGroupName.Length -gt 64){ # AD groups are limited to 64 characters, trim if longer 
            $NumberOfCharactersToRemove = $NumberOfCharactersToRemove + "."
            $TrimmedApplicationName = $ApplicationSimpleName -replace "$NumberOfCharactersToRemove$" # .$ is the last character of the string
            $ADGroupName = ("$Publisher`_$TrimmedApplicationName`_$ApplicationVersion`_$Naming").Replace(" ","")
            Write-Log -LogOutput ("AD group name is too long, 64 characters is the limit. Length = '$($ADGroupName.Length)'. trimming...") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
        }

        $GroupDescription = ("This group links to the SCCM $($type.ToLower()) collection '$CollectionName'")
        $CollectionDescription = ("This collection links to the AD group '$ADGroupName'")

        # Create schedule
        $Schedule = New-CMSchedule -Start (Random-StartTime) -RecurInterval Days -RecurCount 1

        # Create AD group
        try {
            Write-Log -LogOutput ("Creating AD group '$ADGroupName'") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
            New-ADGroup -Name $ADGroupName -Path $OUPath -Description $GroupDescription -GroupScope Universal -ErrorAction SilentlyContinue
        }
        catch {
            Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
        }

        # Create collections and rules
        if($Type -eq "Device"){

            # Write to the log
            Write-Log -LogOutput ("Creating device collection '$CollectionName', assigning rule and moving to folder '$CollectionDevFolderPath'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

            # Create Application if not already existing
            if($AppCollection = Get-CMDeviceCollection -Name $CollectionName){
                
                # The collection already excist
                Write-Log -LogOutput ("Device collection '$CollectionName', already exist") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            }else{

                # Create the collection
                try {
                    Write-Log -LogOutput ("Creating device collection '$CollectionName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                    $AppCollection = New-CMDeviceCollection -Name $CollectionName -LimitingCollectionName $DeviceLimitingCollection -RefreshType Periodic -RefreshSchedule $Schedule -Comment $CollectionDescription
                }
                catch {
                    Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null

                }
            }

            # Create the query rule
            try {
                Write-Log -LogOutput ("Assigning rule for the device collection '$CollectionName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                Add-CMDeviceCollectionQueryMembershipRule -Collection $AppCollection -QueryExpression "select SMS_R_SYSTEM.ResourceID,SMS_R_SYSTEM.ResourceType,SMS_R_SYSTEM.Name,SMS_R_SYSTEM.SMSUniqueIdentifier,SMS_R_SYSTEM.ResourceDomainORWorkgroup,SMS_R_SYSTEM.Client from SMS_R_System JOIN SMS_UserMachineRelationship ON SMS_R_System.Name=SMS_UserMachineRelationship.MachineResourceName JOIN SMS_R_User ON SMS_UserMachineRelationship.UniqueUserName=SMS_R_User.UniqueUserName Where SMS_R_User.UniqueUserName in (select UniqueUserName from SMS_R_User where UserGroupName = ""$DomainNetbiosName\\$ADGroupName"")" -RuleName "Members of AD group $ADGroupName"
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }

            # Move collection
            try {
                Write-Log -LogOutput ("Moving the device collection to the folder '$CollectionUsrFolderPath'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                Move-CMObject -FolderPath $CollectionDevFolderPath -InputObject $AppCollection
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }
        }else{

            # Create Application if not already existing
            if($AppCollection = Get-CMUserCollection -Name $CollectionName){
                
                # The collection already excist
                Write-Log -LogOutput ("User collection '$CollectionName', already exists") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            }else{

                # Create the collection
                Write-Log -LogOutput ("Creating user collection '$CollectionName', assigning rule and moving to folder '$CollectionUsrFolderPath'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                $AppCollection = New-CMUserCollection -Name $CollectionName -LimitingCollectionName $UserLimitingCollection -RefreshType Periodic -RefreshSchedule $Schedule -Comment $CollectionDescription
            }
            
            # Create the query rule
            try {
                Write-Log -LogOutput ("Assigning rule for the user collection '$CollectionName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                Add-CMUserCollectionQueryMembershipRule -Collection $AppCollection -QueryExpression "select SMS_R_USER.ResourceID,SMS_R_USER.ResourceType,SMS_R_USER.Name,SMS_R_USER.UniqueUserName,SMS_R_USER.WindowsNTDomain from SMS_R_User where SMS_R_User.UserGroupName = ""$DomainNetbiosName\\$ADGroupName""" -RuleName "Members of AD group $ADGroupName"
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }

            # Move collection
            try {
                Write-Log -LogOutput ("Moving the user collection to the folder '$CollectionUsrFolderPath'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                Move-CMObject -FolderPath $CollectionUsrFolderPath -InputObject $AppCollection  
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }
        }
    }

    # Create deployment type
    If([array]$MSIfiles = (Get-ChildItem -Path "Microsoft.PowerShell.Core\FileSystem::$SourcePath" -Recurse) | Where-Object{$_.Extension -eq ".msi"}){
        
        # For each MSI file, create detection clause
        ForEach($MSIFile in $MSIfiles){
            do{
                $Failed = $false

                try {
                    [string]$ProductCode = (Get-MsiProperty (($MSIFile.PSPath).Replace("Microsoft.PowerShell.Core\FileSystem::","")) "'ProductCode'")
                    [array]$DetectionClauses += New-CMDetectionClauseWindowsInstaller -ProductCode $ProductCode -Existence
                }
                catch {
                    $Counter ++
                    Write-Log -LogOutput ("Failed to get GUID from the MSI file, might be in use, waiting counter '$counter/30'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                    $Failed = $True
                    Start-sleep -Seconds 5
                }
            }while($Failed -and ($Counter -le 30))
        }

        #Add deploymenttype with MSI detection method
        try {
            Write-Log -LogOutput ("One or more MSIs detected. Creating deployment type with MSI detection method") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            Add-CMScriptDeploymentType -ApplicationName $ApplicationName -DeploymentTypeName "Install $ApplicationName" -ContentLocation $SourcePath -InstallCommand $InstallCommand -UninstallCommand $UninstallCommand -AddDetectionClauses $DetectionClauses -InstallationBehaviorType InstallForSystem -UserInteractionMode Hidden -LogonRequirementType WhereOrNotUserLoggedOn -Force32Bit:$true | Out-Null
        }
        catch {
            Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
        }
    }else{

        #Add deployment type without MSI detection method, create
        try {
            Write-Log -LogOutput ("No MSIs detected. Creating deployment type with HKLM detection method") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            $HKLMDetectionClause = New-CMDetectionClauseRegistryKeyValue -Existence -Hive LocalMachine -KeyName "SOFTWARE\SCCM Auto Management" -ValueName $Collection -PropertyType String
            Add-CMScriptDeploymentType -ApplicationName $ApplicationName -DeploymentTypeName "Install $ApplicationName" -ContentLocation $SourcePath -InstallCommand $InstallCommand -UninstallCommand $UninstallCommand -AddDetectionClauses $HKLMDetectionClause -InstallationBehaviorType InstallForSystem -UserInteractionMode Hidden -LogonRequirementType WhereOrNotUserLoggedOn -Force32Bit:$true | Out-Null
        }
        catch {
            Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
        }
    }

    # Distributing content
    try {
        Write-Log -LogOutput ("Distributing content to the distribution group '$($DistributionPointGroup)'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
        Start-CMContentDistribution -DistributionPointGroupName $DistributionPointGroup -ApplicationName $ApplicationName | Out-Null
    
    }
    catch {
        Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
    }

    # Create deployments
    Foreach($Deployment in $Deployments.GetEnumerator()){

        # Define variables
        $Naming = $Deployment.Value.Naming
        $DeployPurpose = $Deployment.Value.DeploymentPurpose
        $DeployAction = $Deployment.Value.DeployAction
        $CollectionName = ("$Collection`_$Naming")

        # Create deployment
        try {
            Write-Log -LogOutput ("Creating deployment named '$($ApplicationName)' with the deploypurpose '$($DeployPurpose)' and action '$($DeployAction)'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            Start-CMApplicationDeployment -CollectionName $CollectionName -Name $ApplicationName -UserNotification DisplaySoftwareCenterOnly -DeployAction $DeployAction -DeployPurpose $DeployPurpose
        }
        catch {
            Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
        }
    }

    #Update the file to mark it as handled
    Update-ApplicationInfoFile -Path $XMLFilePath -PropertyName "AutoManageAction" -Value ""
    Update-ApplicationInfoFile -Path $XMLFilePath -PropertyName "AutoManageState" -Value "Created"
}

ForEach($Application in $DeleteApplications){

    #Define variables
    $ApplicationName = $Application.Name #the unique name.
    $Publisher = $Application.Publisher
    $ApplicationSimpleName = $Application.SimpleName
    $ApplicationVersion = $Application.Version
    $SourcePath = $Application.SourcePath
    $Collection = ("$Publisher`_$ApplicationSimpleName`_$ApplicationVersion").Replace(" ","")
    $ADGroup = ("$Publisher`_$ApplicationSimpleName`_$ApplicationVersion").Replace(" ","")
    $XMLFilePath = $Application.XMLFilePath

    #Delete collections, AD group and query rule for
    ForEach($Deployment in $Deployments.GetEnumerator()){

        # Define variables
        $Naming = $Deployment.Value.Naming
        $Type = $Deployment.Value.Type
        $CollectionName = ("$Collection`_$Naming") # If you change this, also change in the create deployment part
        $ADGroupName = ("$ADGroup`_$Naming")
        $NumberOfCharactersToRemove = $null
        while($ADGroupName.Length -gt 64){ # AD groups are limited to 64 characters, trim if longer 
            $NumberOfCharactersToRemove = $NumberOfCharactersToRemove + "."
            $TrimmedApplicationName = $ApplicationSimpleName -replace "$NumberOfCharactersToRemove$" # .$ is the last character of the string
            $ADGroupName = ("$Publisher`_$TrimmedApplicationName`_$ApplicationVersion`_$Naming").Replace(" ","")
            Write-Log -LogOutput ("AD group name is too long, 64 characters is the limit. Length = '$($ADGroupName.Length)'. trimming...") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
        }

        # Delete AD group
        try {
            Write-Log -LogOutput ("Deleting AD group '$ADGroupName'") -FunctionName $($MyInvocation.MyCommand)-Path $LogLocation -Name $LogName | Out-Null
            Remove-ADGroup -Identity $ADGroupName -ErrorAction SilentlyContinue -Confirm:$false
        }
        catch {
            Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
        }

        # Delete collections and rules
        if($Type -eq "Device"){

            # Deleteing device collection
            try {
                Write-Log -LogOutput ("Deleteing device collection '$CollectionName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
                Remove-CMCollection -Name $CollectionName -Force -Confirm:$false
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }
        }else{

            # Deleteing device collection
            try {
            Write-Log -LogOutput ("Deleteing user collection '$CollectionName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
            Remove-CMCollection -Name $CollectionName -Force -Confirm:$false
            }
            catch {
                Write-Log -LogOutput ("$_") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName -LogLevel 3 | Out-Null
            }
        }
    }

    #Remove the applicaiton
    Write-Log -LogOutput ("Deleteing application '$ApplicationName'") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null
    Remove-CMApplication -Name $ApplicationName -Force -Confirm:$false

    #Update the file to mark it as handled
    Update-ApplicationInfoFile -Path $XMLFilePath -PropertyName "AutoManageAction" -Value ""
    Update-ApplicationInfoFile -Path $XMLFilePath -PropertyName "AutoManageState" -Value "Deleted"

}

Write-Log -LogOutput ("***********************************************END SCRIPT***********************************************") -FunctionName $($MyInvocation.MyCommand) -Path $LogLocation -Name $LogName | Out-Null

